@@ -58,7 +58,7 @@ class JSONDecoder(json.JSONDecoder):
 
 class Session(object):
 
-	__version__ = "1.2.2-6"
+	__version__ = "1.2.5-2"
 
 	def __init__(self, domain=None, username=None, api_key=None, pwd=None, hostname=None, port=None, proxy=None, verbose=False, dev=False):
 		''' Setup; store credentials, authenticate, get a session key '''
@@ -69,17 +69,17 @@ class Session(object):
 		self._proxy = proxy
 		self._dev = dev
 		self._clearance = CLEARANCE_NONE
-		self.verbose("Creating AccSyn Python API session (v%s)"%Session.__version__)
-		# Migrate
-		# Migrate
+		self.verbose("Creating Accsyn Python API session (v%s)"%Session.__version__)
 		for key in os.environ:
 			if key.startswith("FILMHUB_"):
 				Session.warning("Found old FilmHUB product environment variable '%s', please migrate!"%key)
 		if domain is None:
-			assert ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ),("Please supply your AccSyn domain/organization or set ACCSYN_DOMAIN environment!")
+			if not ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ):
+				raise AccsynException("Please supply your Accsyn domain/organization or set ACCSYN_DOMAIN environment!")
 		self._domain = domain or (os.environ['ACCSYN_DOMAIN'] if 'ACCSYN_DOMAIN' in os.environ else os.environ.get('ACCSYN_ORG', os.environ.get('FILMHUB_DOMAIN', os.environ.get('FILMHUB_ORG'))))
 		if username is None:
-			assert ('ACCSYN_API_USER' in os.environ or 'FILMHUB_API_USER' in os.environ),("Please supply your AccSyn user name (E-mail) or set ACCSYN_API_USER environment!")
+			if not ('ACCSYN_API_USER' in os.environ or 'FILMHUB_API_USER' in os.environ):
+				raise AccsynException("Please supply your Accsyn user name (E-mail) or set ACCSYN_API_USER environment!")
 		self._username = username or os.environ.get('ACCSYN_API_USER') or os.environ['FILMHUB_API_USER']
 		if api_key:
 			self._api_key = api_key
@@ -90,7 +90,7 @@ class Session(object):
 				# Store it temporarily
 				self._pwd = pwd
 			else:
-				raise Exception("Please supply your AccSyn API KEY or set ACCSYN_API_KEY environment!")
+				raise AccsynException("Please supply your Accsyn API KEY or set ACCSYN_API_KEY environment!")
 		self._hostname = hostname
 		self._port = port or ACCSYN_PORT
 		if self._hostname is None:
@@ -136,7 +136,6 @@ class Session(object):
 	@staticmethod
 	def safely_printable(s):
 		return ((s or "").encode()).decode("ascii", "ignore")
-		#return unicodedata.normalize('NFKD', unicode(s)).encode('ascii', 'ignore')
 		#if isinstance(s, str):
 		#	try:
 		#		u = str(s)
@@ -151,7 +150,6 @@ class Session(object):
 	@staticmethod
 	def json_serial(obj):
 		"""JSON serializer for objects not serializable by default json code"""
-
 		if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
 			return obj.isoformat()
 		raise TypeError ("Type %s not serializable" % type(obj))
@@ -161,6 +159,9 @@ class Session(object):
 		''' Return a string representation of a dict '''
 		return json.dumps(d, default=Session.json_serial, indent=indent) if not d is None else ""
 
+	def get_last_message(self):
+		return self._last_message
+
 	@staticmethod
 	def base64_encode(s):
 		return (base64.b64encode((s).encode())).decode("utf-8", "ignore")
@@ -190,9 +191,6 @@ class Session(object):
 		self._uid = result['id']
 		return True
 
-	def get_last_message(self):
-		return self._last_message
-
 	@staticmethod
 	def obscure_dict_string(s):
 		if s is None:
@@ -213,7 +211,7 @@ class Session(object):
 	# REST get
 
 	def event(self, method, uri, data, query=None, entityid=None, timeout=None, ssl=True, quiet=False):
-		''' Generate an Event and send using REST to AccSyn cloud '''
+		''' Generate an Event and send using REST to Accsyn cloud '''
 		assert (self._session_key),("Please login before attempting to post event!")
 		event = {
 			'audience':"api",
@@ -265,25 +263,52 @@ class Session(object):
 		return retval
 
 	def rest(self, method, hostname, uri, data, timeout=None, ssl=True, port=None, quiet=False, headers=None):
-		''' Talk REST with FIlmHUB Cloud '''
+		''' Talk REST with Accsyn Cloud '''
 		if port is None:
 			port = self._port or ACCSYN_PORT
 		if hostname is None:
-			hostname = "%s.%s"%(self._domain, ACCSYN_DOMAIN)
+			hostname = "%s.%s"%(self._domain, ACCSYN_CLOUD_DOMAIN)
 		# Proxy set?
-		proxy = self._proxy or os.environ.get('ACCSYN_PROXY')
-		if 0<len(proxy or ""):
-			if 0<proxy.find(":"):
-				parts = proxy.split(":")
-				hostname = parts[0]
-				port = int(parts[1])
+		proxy_type = None
+		proxy_hostname = None
+		proxy_port = -1
+		proxy_setting = self._proxy or os.environ.get('ACCSYN_PROXY')
+		if 0<len(proxy_setting or ""):
+			if 0<proxy_setting.find(":"):
+				parts = proxy_setting.split(":")
+				if len(parts) == 3:
+					proxy_type = parts[0]
+					proxy_hostname = parts[1]
+					proxy_port = int(parts[2])
+				elif len(parts) == 2:
+					proxy_type = "accsyn"
+					proxy_hostname = parts[0]
+					proxy_port = int(parts[1])
 			else:
-				hostname = proxy
-				port = 80
+				proxy_type = "accsyn"
+				proxy_hostname = proxy
+				proxy_port = 80
 			ssl = False
-		elif self._dev and (uri or "").find("registry") != 0:
+		if self._dev and (uri or "").find("registry") != 0:
 			ssl = False
 			port = 80
+		if proxy_type == "accsyn":
+			if proxy_port == -1:
+				proxy_port = 80
+			self.verbose("Using Accsyn proxy @ %s:%s"%(proxy_hostname, proxy_port))
+			hostname = proxy_hostname
+			port = proxy_port
+		elif proxy_type in ["socks","socks5"]:
+			try:
+				self.verbose("Using SOCKS5 proxy @ %s:%s"%(proxy_hostname, proxy_port))
+				import socks
+				socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, proxy_hostname, proxy_port)
+				socket.socket = socks.socksocket
+			except ImportError as ie:
+				Session.warning("Python lacks SOCKS support, please install 'pysocks' and try again...")
+				raise ie
+		elif not proxy_type is None:
+			raise AccsynException("Unknown proxy type '%s'!"%proxy_type)
 		url = "http%s://%s:%d/api/v1.0%s"%("s" if ssl else "", hostname, port,("/" if not uri.startswith("/") else "")+uri)
 		if timeout is None:
 			timeout = 999999
@@ -293,11 +318,11 @@ class Session(object):
 		#r = requests.post(github_url, data, auth=('user', '*****'))
 		CONNECT_TO, READ_TO = (10, 2*60)  # Wait 10s to reach machine, 2min for it to send back data
 		r = None
-		#data = AccSyn.prepare_rest_serialize(data)
+		#data = Accsyn.prepare_rest_serialize(data)
 		initial_timeout = timeout
 		if headers is None:
 			if uri.find("registry/")!=0:
-				assert (not self._session_key is None),("Need to be authenticated when communicating with AccSyn!")
+				assert (not self._session_key is None),("Need to be authenticated when communicating with Accsyn!")
 				headers = {'Authorization':"ASSession %s"%(Session.base64_encode('{"domain":"%s","username":"%s","session_key":"%s"}'%(self._domain, self._username, self._session_key)))}
 			else:
 				headers = {}
@@ -322,13 +347,13 @@ class Session(object):
 				timeout -= int((t_end-t_start)/1000)
 				timeout -= sleep_time
 				if timeout<=0 or True:
-					raise Exception("Could not reach %s:%d! Make sure cloud server(%s) can be reached from you location and no firewall is blocking outgoing traffic at port %s. Details: %s"%(hostname, port, port ,hostname,traceback.format_exc() if not quiet else "(quiet)"))
+					raise AccsynException("Could not reach %s:%d! Make sure cloud server(%s) can be reached from you location and no firewall is blocking outgoing traffic at port %s. Details: %s"%(hostname, port, port ,hostname,traceback.format_exc() if not quiet else "(quiet)"))
 					
 				Session.warning("Could not reach %s:%d! Waited %ds/%s, will try again in %ds... Details: %s"%(hostname, port, initial_timeout-timeout, "%ss"%initial_timeout if initial_timeout<99999999 else "INF", sleep_time, traceback.format_exc()))
 				time.sleep(sleep_time)
 
 			try:
-				#retval = AccSyn.prepare_rest_deserialize(r.json())
+				#retval = Accsyn.prepare_rest_deserialize(r.json())
 				retval = json.loads(r.text, cls=JSONDecoder)
 				if not quiet:
 					self.verbose("%s/%s REST %s result of %s: %s (~%sms)"%(
@@ -357,17 +382,17 @@ class Session(object):
 				print(traceback.format_exc(), file=sys.stderr)
 				message = "The %s:%d/%s REST %s %s operation failed! Details: %s %s"%(hostname, port, uri, method, Session.obscure_dict_string(Session.safely_printable(str(data)).replace("'",'"')), r.text, traceback.format_exc())
 				Session.warning(message)
-				raise Exception(message)
+				raise AccsynException(message)
 		if 'exception' in retval:
 			message = "%s caused an exception! Please contact %s admin for more further support."%(uri, self._domain)
 			Session.warning(message)
 			if self._clearance in [CLEARANCE_ADMIN, CLEARANCE_CLOUDADMIN]:
 				Session.warning(retval['exception'])
-			raise Exception(message)
+			raise AccsynException(message)
 		elif 'message' in retval:
 			message_effective = retval.get('message_hr') or retval['message'] 
 			Session.warning(message_effective)
-			raise Exception(message_effective)
+			raise AccsynException(message_effective)
 		return retval
 
 	def decode_query(self, query):
@@ -401,7 +426,7 @@ class Session(object):
 			else:
 				is_at_whitespace = False
 				if query[idx] == '"' and is_escaped:
-					is_at_whitespace = False
+					is_escaped = not is_escaped
 				elif query[idx] == "(":
 					if not is_escaped:
 						paranthesis_depth += 1
@@ -454,7 +479,7 @@ class Session(object):
 				if os.path.exists(data):
 					data = json.load(open(data, "r"))
 				else:
-					raise Exception("Cannot build JSON payload data, not a valid JSON string or path to a JSON file!")
+					raise AccsynException("Cannot build JSON payload data, not a valid JSON string or path to a JSON file!")
 		else:
 			assert (not data is None and 0<len(data)),("Empty create data submitted!")
 
@@ -467,9 +492,9 @@ class Session(object):
 
 	# Query
 
-	def find(self, query, attributes=None, archived=False, limit=None, skip=None):
+	def find(self, query, attributes=None, finished=None, limit=None, skip=None):
 		''' Return a list of something '''
-		assert (0<len(query or "") and (isinstance(query, str) or isinstance(query, str))),("Invalid query type supplied, must be of string type!")
+		assert (0<len(query or "") and (isinstance(query, str) or isinstance(query, str)  or isinstance(query, unicode))),("Invalid query type supplied, must be of string type!")
 		retval = None
 		d = self.decode_query(query)
 		data = {}
@@ -494,8 +519,8 @@ class Session(object):
 				data = {'type':2}
 			elif d['entitytype'] == "job":
 				data = {'type':1}
-			if archived:
-				data['archive']=True
+			if not finished is None:
+				data['finished']=finished
 			if limit:
 				data['limit'] = limit
 			if skip:
@@ -532,19 +557,30 @@ class Session(object):
 		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, str))),("Invalid entity type supplied, must be of string type!")
 		assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, str))),("Invalid entity ID supplied, must be of string type!")
 		assert (0<len(data or {}) and isinstance(data, dict)),("Invalid data supplied, must be dict and have content!")
-		d = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
-		if d:
-			return d['result']
+		response = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
+		if response:
+			return response['result'][0]
+
+	def update_many(self, entitytype, data, entityid=None):
+		''' Update many entities '''
+		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, str) or isinstance(entitytype, unicode))),("Invalid entity type supplied, must be of string type!")
+		assert (entitytype.lower() == "task"),("Only multiple 'task' entities can be updated!")		
+		if entitytype.lower() == "task":
+			assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, str) or isinstance(entityid, unicode))),("Invalid entity ID supplied, must be of string type!")
+		assert (0<len(data or []) and isinstance(data, list)),("Invalid data supplied, must be a list!")
+		response = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
+		if response:
+			return response['result']
 
 	# Delete an entity
 
 	def delete_one(self, entitytype, entityid):
 		''' Update an entity '''
-		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, str))),("Invalid entity type supplied, must be of string type!")
-		assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, str))),("Invalid entity ID supplied, must be of string type!")
-		d = self.event("DELETE", "%s/delete"%Session.get_base_uri(entitytype), {}, entityid=entityid)
-		if d:
-			return d['result'][0]
+		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, str) or isinstance(entitytype, unicode))),("Invalid entity type supplied, must be of string type!")
+		assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, str) or isinstance(entityid, unicode))),("Invalid entity ID supplied, must be of string type!")
+		response = self.event("DELETE", "%s/delete"%Session.get_base_uri(entitytype), {}, entityid=entityid)
+		if response:
+			return response['result']
 
 	# File operations
 	def ls(self, p, recursive=False, maxdepth=None, files_only=False, directories_only=False):
@@ -561,9 +597,9 @@ class Session(object):
 			data['directories_only'] = directories_only
 		if files_only:
 			data['files_only'] = files_only
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
 
 	def getsize(self, p):
 		assert (0<len(p or "") and (isinstance(p, str) or isinstance(p, str) or isinstance(p, dict) or isinstance(p, list))),("No path supplied, or not a string/list!")
@@ -571,9 +607,9 @@ class Session(object):
 			'op':"getsize",
 			'path':p,
 		}
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
 
 	def exists(self, p):
 		assert (0<len(p or "") and (isinstance(p, str) or isinstance(p, str) or isinstance(p, dict) or isinstance(p, list))),("No path supplied, or not a string/list!")
@@ -581,9 +617,28 @@ class Session(object):
 			'op':"exists",
 			'path':p,
 		}
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
+
+	# Pre publish
+	def prepublish(self, data):
+		if data is None or not isinstance(data, list):
+			raise AccsynException("None or empty data supplied!")
+		# Check entries, calculate size
+		def recursive_get_size(files):
+			result = 0
+			for d in files:
+				if not 'size' in d:
+					d['size'] = 0 if not d.get('is_dir') is True or d.get('files') is None else recursive_get_size(d['files'])
+				result += d['size']
+			return result
+		event_data = {
+			'files':data,
+			'size':recursive_get_size(data)
+		}
+		response = self.event("PUT", "organization/publish/preprocess", event_data)
+		return response['result']
 
 	# Misc
 	def get_api_key(self):
@@ -611,8 +666,8 @@ class Session(object):
 
 	# Help
 	def help(self):
-		print("Please have a look at the Python API reference: https://sites.google.com/accsyn.com/python-api")
-
-
-
+		print("Please have a look at the Python API reference: https://support.accsyn.com/python-api")
 
+class AccsynException(Exception):
+	def __init__(self, message):
+		super(AccsynException, self).__init__(message)
